-- See the paths for the customers acquired through all channels
-- This experiment is to visualize the results as Sankey
-- Filtering events so that results can be visualized
-- Observing first 4 events for these channels
DROP TABLE IF EXISTS onboarding.all_acquisition_channel_npath_4_events;
CREATE TABLE onboarding.all_acquisition_channel_npath_4_events
    DISTRIBUTE BY HASH(cmusno) 
    COMPRESS AS (
    SELECT *
    FROM NPATH(
    ON (SELECT *
    FROM onboarding.event_list_18_0_to_90_removed
    WHERE domain_type IN ('acq_chn', 'APPLICATION', 'AUTO_PAYMENT', 'CREDIT_CARD', 'CreditCard', 'DEPOSIT', 'FUND_TRANSFER', 'Investment', 'LOAN', 'Loans', 
    'CALL_CENTER', 'COMPLAINT'))
    PARTITION BY cmusno
    ORDER BY row_number
    MODE('NONOVERLAPPING')
    SYMBOLS(domain_type IN ('acq_chn') AS a,
    domain_type NOT IN('acq_chn') AS b)
    PATTERN('a.b.b?.b?.b?')  -- see up to first 4 events of customers
    RESULT(ACCUMULATE(event_desc OF ANY(a,b)) AS path,
    FIRST(cmusno OF a) AS cmusno,
    COUNT(cmusno OF ANY(a)) AS lenPath,
    FIRST(event_date OF ANY(b)) AS start_date,
    LAST(event_date OF ANY(b)) AS end_date)
    )
    );
GRANT ALL ON onboarding.all_acquisition_channel_npath_4_events TO db_analytics;

DROP TABLE IF EXISTS onboarding.freq_all_acquisition_channel_npath_4_events;
CREATE TABLE onboarding.freq_all_acquisition_channel_npath_4_events
    DISTRIBUTE BY HASH(path) 
    COMPRESS AS (
    SELECT path, CAST(avg( CAST (end_date AS DATE) - CAST (start_date AS DATE) ) AS INTEGER) duration, lenPath, count(*) cnt
    FROM onboarding.all_acquisition_channel_npath_4_events
    GROUP BY 1, lenPath
    );
GRANT ALL ON onboarding.freq_all_acquisition_channel_npath_4_events TO db_analytics;

SELECT count (DISTINCT cmusno)
    FROM onboarding.event_list_18_0_to_90; --86593

-- Aggregated Path Counts
SELECT count (*)
    FROM onboarding.freq_all_acquisition_channel_npath_4_events; -- 6922 Paths

-- Single user paths
SELECT count(*)
    FROM onboarding.freq_all_acquisition_channel_npath_4_events
    WHERE cnt = 1; --3698 individual paths

-- Customers having common paths equal to 10 or more
SELECT sum(cnt)
    FROM onboarding.freq_all_acquisition_channel_npath_4_events
    WHERE cnt >= 3; --79858

--AppCenter Visualization Code for all channels
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_all_acquisition_channel_npath_4_events where cnt >= 10 ) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('sankey')
--);

--AppCenter Visualization Code for SUBE
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_all_acquisition_channel_npath_4_events where path like '%' || '0_SUBE' || '%' and cnt >= 5) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('sankey')
--);

--AppCenter Visualization Code for DIREK_SATIS
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_all_acquisition_channel_npath_4_events where path like '%' || '0_DIREK_SATIS' || '%' and cnt >= 3) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('sankey')
--);

-- AppCenter Visualization code for MAAS
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_all_acquisition_channel_npath_4_events where path like '%' || '0_MAAS' || '%' and cnt >= 3) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('sankey')
--);
