-- Check the overall application rate for all the acquisition channels
-- Only keeping application domain to visualize paths
DROP TABLE IF EXISTS onboarding.application_rate_npath;
CREATE TABLE onboarding.application_rate_npath
DISTRIBUTE BY HASH(cmusno) 
COMPRESS AS (
		SELECT * 
		FROM npath(
		ON (select * from onboarding.event_list_18_0_to_90_removed
			where domain_type in ('acq_chn', 'APPLICATION') AND event_desc NOT IN ('UnknownApp_', 'UnknownApp_Rejctd', 'UnknownApp_Apprvd'))
		PARTITION BY cmusno
		ORDER BY row_number
		MODE('NONOVERLAPPING')
		SYMBOLS(domain_type in ('acq_chn') as a,
				domain_type in ('APPLICATION') as b)
		PATTERN('a.b+')  -- All application events
		RESULT(ACCUMULATE(event_desc of any(a, b)) as path,
			   FIRST(cmusno of a) as cmusno,
			   COUNT(cmusno of any(a, b)) as lenPath,
			   FIRST(event_date of a) as start_date,
			   LAST(event_date of b) as end_date)
	)
);
grant all on onboarding.application_rate_npath to db_analytics;

DROP TABLE IF EXISTS onboarding.freq_application_rate_npath;
CREATE TABLE onboarding.freq_application_rate_npath
DISTRIBUTE BY HASH(path)
COMPRESS AS (
	select path, cast(avg( cast (end_date as date) - cast (start_date as date) ) as integer) duration, lenPath, count(*) cnt
	from onboarding.application_rate_npath
	where lenPath >= 2
	group by 1, lenPath
);
grant all on onboarding.freq_application_rate_npath to db_analytics;

-- All Paths count
select count (*) from onboarding.application_rate_npath;   --73838 customers go for 1 or more applications
select count (distinct cmusno) from onboarding.application_rate_npath; --73838

-- See the Unique Paths
select count (*) from onboarding.freq_application_rate_npath; -- 18612 Unique Paths

-- Single user paths
select count(*) from onboarding.freq_application_rate_npath where cnt = 1; --14766 individual paths

-- Customers having common paths equal to 2 or more
SELECT sum(cnt)
    FROM onboarding.freq_application_rate_npath
    WHERE cnt >= 5; --52798

--Customers where path length max path length is 8
SELECT sum(cnt)
    FROM onboarding.freq_application_rate_npath
    WHERE lenpath <= 8; --46344

-- Visualization Code for AppCenter
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_application_rate_npath where cnt >= 5) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('tree')
--);