DROP TABLE IF EXISTS onboarding.open_ccenter_npath;
CREATE TABLE onboarding.open_ccenter_npath
DISTRIBUTE BY HASH(cmusno) 
COMPRESS AS (
		SELECT * 
		FROM npath(
		ON (select * from onboarding.event_list_18_0_to_90_removed
			where domain_type in ('acq_chn', 'CALL_CENTER') or upper(event_desc) like '%OPEN%')
		PARTITION BY cmusno
		ORDER BY row_number
		MODE('NONOVERLAPPING')
		SYMBOLS(domain_type in ('acq_chn') as a,
				upper(event_desc) like '%OPEN%' as b,
				event_desc in ('CALL_CENTER') as c,
				true as d)
		PATTERN('a.b+.c+.d*')  
		RESULT(ACCUMULATE(event_desc of any(a, b, c, d)) as path,
			   FIRST(cmusno of a) as cmusno,
			   COUNT(cmusno of any(a, b,c,d)) as lenPath,
			   FIRST(event_date of a) as start_date,
			   LAST(event_date of d) as end_date)
	)
);
grant all on onboarding.open_ccenter_npath to db_analytics;

DROP TABLE IF EXISTS onboarding.freq_open_ccenter_npath;
CREATE TABLE onboarding.freq_open_ccenter_npath
DISTRIBUTE BY HASH(path)
COMPRESS AS (
	select path, cast(avg( cast (end_date as date) - cast (start_date as date) ) as integer) duration, lenPath, count(*) cnt
	from onboarding.open_ccenter_npath
	where lenPath >= 2
	group by 1, lenPath
);
grant all on onboarding.freq_open_ccenter_npath to db_analytics;

-- All Paths count
select count (*) from onboarding.open_ccenter_npath;   --60260 customers involved with call center

select count (distinct cmusno) from onboarding.open_ccenter_npath; --60260

-- See the Unique Paths
select count (*) from onboarding.freq_open_ccenter_npath; -- 25879 Unique Paths

-- Single user paths
select count(*) from onboarding.freq_open_ccenter_npath where cnt = 1; --22780 individual paths

-- Customers having common paths equal to 2 or more
SELECT sum(cnt)
    FROM onboarding.freq_open_ccenter_npath
    WHERE cnt >= 2; --37480

--Customers where path length max path length is 8
SELECT sum(cnt)
    FROM onboarding.freq_open_ccenter_npath
    WHERE lenpath <= 8; --44467

-- Visualization Code for AppCenter

INSERT INTO app_center_visualizations  (json) 
SELECT json FROM Visualizer (
    ON (select * from onboarding.freq_open_ccenter_npath where cnt >= 2) PARTITION BY 1 
AsterFunction('npath') 
Title('44-marketing_to_marketing_revoked') 
VizType('tree'));