-- See all the customers with Autopayments
DROP TABLE IF EXISTS onboarding.autopay_payment_npath;
CREATE TABLE onboarding.autopay_payment_npath
    DISTRIBUTE BY HASH(cmusno) 
    COMPRESS AS (
    SELECT *
    FROM NPATH(
    ON (SELECT *
    FROM onboarding.event_list_18_0_to_90_removed
    WHERE   domain_type IN ('acq_chn') or event_desc in ('AutoPmtOpen_OFO','AutoPmtCancel_OFO','TrxTb_CorporationPay_BillPay'))
    PARTITION BY cmusno
    ORDER BY row_number
    MODE('NONOVERLAPPING')
    SYMBOLS(domain_type IN ('acq_chn') AS a,
    true as b
    )
    PATTERN('a.b+')  -- Get all the customers where there is one autopayment event
    RESULT(ACCUMULATE(event_desc OF ANY(a,b)) AS path,
    FIRST(cmusno OF a) AS cmusno,
    COUNT(cmusno OF ANY(a, b)) AS lenPath,
    FIRST(event_date OF a) AS start_date,
    LAST(event_date OF ANY(a,b)) AS end_date)
    )
    );
GRANT ALL ON onboarding.autopay_payment_npath TO db_analytics;

--Aggregate paths, calculate average duration for events and count the paths
DROP TABLE IF EXISTS onboarding.freq_autopay_payment_npath;
CREATE TABLE onboarding.freq_autopay_payment_npath
    DISTRIBUTE BY HASH(path)
    COMPRESS AS (
    SELECT path, CAST(avg( CAST (end_date AS DATE) - CAST (start_date AS DATE) ) AS INTEGER) duration, lenPath, count(*) cnt
    FROM onboarding.autopay_payment_npath
    WHERE lenPath >= 2
    GROUP BY 1, lenPath
    );
GRANT ALL ON onboarding.freq_autopay_payment_npath TO db_analytics;



    
-- Visualization Code for AppCenter
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_autopay_payment_npath  ) PARTITION BY 1 
--AsterFunction('npath') 
--Title('55-autopay_payment_npath') 
--VizType('tree'));